#!/usr/bin/env lua
local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

coaf.ChDir(coaf.GetAppDir(arg))

local socket = require "cosocket.core"
local client = require "coaf.client"
local adapter = require "coaf.cli_adapter.simple"
local service = require "coaf.service"

local event = require "coaf.event"

local withLog = false
local _x
for _x=1, #arg do
	if arg[_x] == "--log" then
		withLog = true
		break
	end
end

if withLog then
	local roll = require "coaf.log.rollfile"
	os.execute( "/bin/mkdir -p /var/log/" )
	local logF = roll.Create("/var/log/PtzManager.log")
	if not logF or not logF:init() then
		_dbg.Error("PtzManager", "Fail to open log file: /var/log/PtzManager.log, so the log messages should be output to console.")
		withLog = false
	else
		_dbg.AddLogger("logfile", logF, nil, nil)
	end
end

coaf.Init( "PtzManager" )
if withLog then
	local stdio = require "coaf.log.stdio"
	stdio.SetDefaultFilterName( "PtzManager" )
	stdio.Start()
end

service.InitSocket()
client.InitSocket( socket, adapter )

_G.cfg_ptz = require("configs.ptz_configs").Load()



_G.g_event = event.createNew( "PtzManager" )

local mod_PtzManager = require "mod_PtzManager"
local cosvc_PtzManager


local function Main(args)
	cosvc_PtzManager = service.New( "PtzManager", "unix:/var/run/PtzManager.sock" )
	if cosvc_PtzManager then
		_dbg.Info( "PtzManager", "CoService of PtzManager is created." )
		cosvc_PtzManager:CreateCoServerProxy()
		cosvc_PtzManager:ExposeObject("/ptz", mod_PtzManager )

	else
		_dbg.Error( "PtzManager", "Fail created CoService!" )
	end

end

coaf.Start(Main, arg)



_dbg.Info("PtzManager", "PtzManager is destroying." )


event.free( "PtzManager" )

_dbg.Print( "PtzManager", "Service terminated." )







