----------------------------------------------------------------------------------------------------
-- AIC3104 chip controller.
-- Copyright (C) 2016, KILOVIEW Elecontrics Co., Ltd.
--
-- Author:  2019/6/11
-----------------------------------------------------------------------------------------------------

local coaf=require "coaf.core"
local _dbg = require "coaf.debug"

local periphery = require "periphery"
local I2C_REG = require "chips.i2c_reg"
local GPIO = periphery.GPIO

------------------------------------------------------
-- The factory function to create a AIC3104 handle object
------------------------------------------------------
local function CreateAIC3104( i2c_dev, rxAddr, lineIO )
        ------------------------------------------------------
        -- Object
        ------------------------------------------------------
        local AIC3104 = {
                i2c_reg = nil,
                plug_st = false, 
                last_plug_st = false,
                FSA8049_EN = GPIO( 920, "out" ),
                HP_DET = GPIO( 921, "in" ),
                workMode = "MICIN"
        }

        if lineIO then
                AIC3104.LINE_INOUT_SEL = GPIO(lineIO, "out")
        end

        ------------------------------------------------------
        i2c_dev = i2c_dev or "/dev/i2c-0"
        
        local BASE_MAP = rxAddr or 0x18

        _dbg.Info("AIC3104", "Creating AIC3104 control object at 0x" .. string.format( "%02x", BASE_MAP) .. "@" .. i2c_dev .. "..." )

        AIC3104.i2c_reg = I2C_REG.Create()
        if not AIC3104.i2c_reg:Open( i2c_dev ) then
                return nil, "Fail to open I2C device"
        end

        -------------------------------------------------------
        -- Object method: Register read
        -------------------------------------------------------
        function AIC3104:ReadReg( map, addr, itemArr )
                return self.i2c_reg:ReadReg(map, addr, itemArr)
        end

        ------------------------------------------------------
        -- Register write
        ------------------------------------------------------
        function AIC3104:WriteReg( map, addr, itemArr )
                return self.i2c_reg:WriteReg(map, addr, itemArr)
        end
        ------------------------------------------------------
        -- Soft reset the chip!
        ------------------------------------------------------
        
        function AIC3104:Reset()        --Soft reset    
                self.FSA8049_EN:write(0)
                if self.LINE_INOUT_SEL and (self.workMode == "LINEIN" or self.workMode == "LINEOUT") then
                        if self.workMode == "LINEOUT" then
                                self.LINE_INOUT_SEL:write(1)
                        else
                                self.LINE_INOUT_SEL:write(0)
                        end
                end
                self:Config()
        end     

        function AIC3104:Config()       --Soft reset    
                --Reset
                self:WriteReg( BASE_MAP, 1 , {0x80} )
                --I2s Config
                self:WriteReg( BASE_MAP, 8 , {0xc0} )
                self:WriteReg( BASE_MAP, 9 , {0x30} )           
                --Mic Input Config
                self:WriteReg( BASE_MAP, 25, {0x80} )  --MICBIAS output is powered to 2.5 V
                self:WriteReg( BASE_MAP, 13, {0x00} )  --0x80:en 0x00:dis Headset / Button Enable                               

                self:WriteReg( BASE_MAP, 12, {0x00} )                           
                
                if self.workMode == "LINEIN" then
                        self:WriteReg( BASE_MAP, 19, {0x04} )  --LINE1LP TO Left-ADC channel and powered up
                        self:WriteReg( BASE_MAP, 22, {0x04} )  --LINE1RP TO Right-ADC channel and powered up                
                        self:WriteReg( BASE_MAP, 15, {0x0a} )  --Left-ADC PGA Gain 0x00
                        self:WriteReg( BASE_MAP, 16, {0x0a} )  --Right-ADC PGA Gain 0x00
                elseif self.workMode == "MICIN" then
                        self:WriteReg( BASE_MAP, 19, {0x7c} )  --Left-ADC channel is powered up
                        self:WriteReg( BASE_MAP, 22, {0x7c} )  --Right-ADC channel is powered up                
                        self:WriteReg( BASE_MAP, 17, {0xf0} )  --To Left-ADC F0-F8 0db to -12db
                        self:WriteReg( BASE_MAP, 18, {0xf0} )  --To Right-ADC F0-F8 0db to -12db                
                        self:WriteReg( BASE_MAP, 15, {0x00} )  --Left-ADC PGA Gain 0x00 0x7f
                        self:WriteReg( BASE_MAP, 16, {0x00} )  --Right-ADC PGA Gain 0x00 0x7f
                        
                        self:WriteReg( BASE_MAP, 26, {0x87} )  -- Left AGC is enabled
                        self:WriteReg( BASE_MAP, 27, {0x7c} )  -- Left-AGC Maximum Gain Allowed
                        self:WriteReg( BASE_MAP, 28, {0x7f} )  -- Noise Gate Hysteresis Level Control--0x02 0x00
                        self:WriteReg( BASE_MAP, 29, {0x87} )  -- Right AGC is enabled
                        self:WriteReg( BASE_MAP, 30, {0x7c} )  -- Right AGC Maximum Gain Allowed
                        self:WriteReg( BASE_MAP, 31, {0x7f} )  -- Noise Gate Hysteresis Level Control--0x02 0x00
                        self:WriteReg( BASE_MAP, 34, {0x7f} )
                        self:WriteReg( BASE_MAP, 35, {0x7f} )
                end

                -- Audio output Config
                self:WriteReg( BASE_MAP, 7 , {0x0a} )
                self:WriteReg( BASE_MAP, 43, {0x00} )
                self:WriteReg( BASE_MAP, 44, {0x00} )
                self:WriteReg( BASE_MAP, 37, {0xc0} )

                        --low pass 435Hz 0db
                        --[[
                        self:WriteReg( BASE_MAP, 0,  {0x01} )  --Switch to Register Page 1

                        self:WriteReg( BASE_MAP, 1,  {0x03} )  --Left N0 MSB
                        self:WriteReg( BASE_MAP, 2,  {0x8c} )  --Left N0 LSB
                        self:WriteReg( BASE_MAP, 3,  {0x03} )  --Left N1 MSB
                        self:WriteReg( BASE_MAP, 4,  {0x8c} )  --Left N1 LSB
                        self:WriteReg( BASE_MAP, 13, {0x78} )  --Left D1 MSB
                        self:WriteReg( BASE_MAP, 14, {0xe6} )  --Left D1 LSB

                        self:WriteReg( BASE_MAP, 27,  {0x03} )  --Right N0 MSB                                                 
                        self:WriteReg( BASE_MAP, 28,  {0x8c} )  --Right N0 LSB                                                 
                        self:WriteReg( BASE_MAP, 29,  {0x03} )  --Right N1 MSB                                                 
                        self:WriteReg( BASE_MAP, 30,  {0x8c} )  --Right N1 LSB                                                 
                        self:WriteReg( BASE_MAP, 39,  {0x78} )  --Right D1 MSB                                                 
                        self:WriteReg( BASE_MAP, 40,  {0xe6} )  --Right D1 LSB                                                 

                        self:WriteReg( BASE_MAP, 0, {0x00} )  --Switch back to page 0 
                        ]]              

                
                self:WriteReg( BASE_MAP, 47, {0x80} )      -- Left  Analog Volume reduction control
                self:WriteReg( BASE_MAP, 64, {0x80} )      -- Right Analog Volume reduction control     
                self:WriteReg( BASE_MAP, 51, {0x49} )      -- Left  Analog Volume up control
                self:WriteReg( BASE_MAP, 65, {0x49} )      -- Right Analog Volume up control



        end

        ------------------------------------------------------
        -- A step handler for AIC3104 processing.
        -- To fit the LUA working mode, you need call the 
        ------------------------------------------------------
        function AIC3104:SetWorkMode(mode)
                mode = mode or "MICIN"
                self.workMode = mode:upper()
                self:Reset()
        end

        ------------------------------------------------------
        -- A step handler for AIC3104 processing.
        -- To fit the LUA working mode, you need call the 
        -- 'Step' periodically ( suggest per 200~500ms).
        -- This routine should do :
        --      *       Signal status checking
        --      *       Auto format detecting
        --      *       Configue the chip to make it output
        --
        ------------------------------------------------------
        
        function AIC3104:Step()
                self.plug_st = self.HP_DET:read()
                if self.plug_st and not self.last_plug_st then
                        --self:WriteReg( BASE_MAP, 1 , {0x80} )
                        coaf.Sleep(0.5) 
                        self.FSA8049_EN:write(0)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(1)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(0)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(1)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(0)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(1)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(0)
                        coaf.Sleep(0.005) 
                        self.FSA8049_EN:write(1)
                        coaf.Sleep(0.5) 

                        if self.workMode == "LINEIN" then 
                                self:WriteReg( BASE_MAP, 19, {0x04} )  
                                self:WriteReg( BASE_MAP, 22, {0x04} )  
                        else
                                self:WriteReg( BASE_MAP, 19, {0x7c} )  --Left-ADC channel is powered up
                                self:WriteReg( BASE_MAP, 22, {0x7c} )  --Right-ADC channel is powered up                
                        end
                        --print("plug_in")
                        elseif not self.plug_st and self.last_plug_st then
                        --coaf.Sleep(500) 
                        self.FSA8049_EN:write(0)
                        self:WriteReg( BASE_MAP, 19, {0x78} )  --Left-ADC channel is powered down?
                        self:WriteReg( BASE_MAP, 22, {0x78} )  --Right-ADC channel is powered down?
                        --self:WriteReg( BASE_MAP, 1 , {0x80} )
                        --print("plug_out")
                end
                self.last_plug_st = self.plug_st
                --print(self.plug_st)
        end
        
        AIC3104:Reset()
        return AIC3104

end --END of CreateSIL9136()

return CreateAIC3104