#!/usr/bin/env lua
local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local thread = require "coaf.thread"
local service = require "coaf.service"
local roll = require "coaf.log.rollfile"

-- Change current dir to app location
coaf.ChDir(coaf.GetAppDir(arg))

local downloader = require "downloader"
local downmap = require "downmap"

------------------------------------------------

-- First, initialize the log file.
os.execute( "/bin/mkdir -p /var/log/downloader" )
local logF = roll.Create("/var/log/downloader/downloader.log")
if not logF or not logF:init() then
	_dbg.Error("DOWNLOAD", "Fail to open log file: /var/log/downloader/downloader.log, so the log messages should be output to console.")
else
	_dbg.AddLogger("logfile", logF, nil, nil)
end

------------------------------------------------

local g_service

local downloadService = {}

------------------------------------------------

function downloadService.StartDownload( down_type, url, name )
	_dbg.Info( "DOWNLOAD", "Starting download: ", down_type, url, name )
	if not down_type then
		_dbg.Error( "DOWNLOAD", "Invalid download type!" )
		return false, "Invalid download type"
	end
	local mapItem = downmap[down_type]
	if not mapItem then
		_dbg.Error( "DOWNLOAD", "Unsupport download type of:", down_type )
		return false, "Unsupport download type of: " .. down_type
	end
	
	if mapItem.filterUrl then
		url = mapItem.filterUrl(url)
	end
	
	if not url then
		_dbg.Error( "DOWNLOAD", "Invalid download URL" )
		return false, "Invalid download URL"
	end

	if not name then
		name = down_type .. ".bin"	-- NOTE: this is !!NOT!! filename, but only an identifier.
	end

	local r,err = downloader.StartDownload(down_type, name, url, mapItem.location)
	if r then
		_dbg.Info( "DOWNLOAD", "Download task is started, task name is: ", name )
	else
		_dbg.Error( "DOWNLOAD", "Start download fail with error: ", err )
	end
	return r,err
end

----

function downloadService.AbortDownload( down_type, name )
	if not down_type then
		return false, "Invalid download type"
	end
	if not name then
		name = down_type .. ".bin"	-- NOTE: this is !!NOT!! filename, but only an identifier.
	end
	_dbg.Info( "DOWNLOAD", "Abort download task: ", name )

	return downloader.AbortDownload(name)
end

----

function downloadService.GetDownloadStatus( down_type, name )
	if not down_type then
		return false, "Invalid download type"
	end
	if not name then
		name = down_type .. ".bin"	-- NOTE: this is !!NOT!! filename, but only an identifier.
	end
	local status = downloader.GetStatus(name)
	if not status then
		return nil
	else
		return { status=status.status, progress=status.progress, message=status.message }
	end
end

---------------------------------------------------------

coaf.Init( "downloader" )
-- Initialize service.
service.InitSocket()

---------------------------------------------------------

local function Main(args)
	downloader.Init()
	g_service = service.New( "downloader", "unix:/var/run/downloader.sock" )
	if g_service then
		g_service:CreateCoServerProxy()
		g_service:ExposeObject("/", downloadService )
	else
		_dbg.Error( "DOWNLOAD", "Fail to create CoService for downloader" )
	end
end

coaf.Start(Main,arg)

_dbg.Print( "DOWNLOAD", "Downloader service is terminated." )

