local DC = require "supertable.declare"
local CONF = require "supertable.core"
local funcs = require "nm_funcs"

local _M = {
	PATCHING = false, --Mark it's on patching or not.
	config_file = "/etc/network/nm.cfg"
}

--------------------------------------------------------

-- Callback when ethernets item changed.
local function on_etherChanged(cfg, ifname)
	if _M.PATCHING then
		return
	end

	funcs.ConfigEthernet(ifname,cfg)
	local root = cfg.__proto.root
	if root and root.aliases and root.aliases[ifname] then
		funcs.ConfigEthernetAlias(ifname, root.aliases[ifname])
	end
end

-- Callback when network alias item changed.
local function on_aliasChanged(cfg, ifname)
	if _M.PATCHING then
		return
	end

	funcs.ConfigEthernetAlias(ifname,cfg)
end

-- Callback when interlink changed.
local function on_interlinkChanged(cfg)
	if _M.PATCHING then
		return
	end

	funcs.ConfigInterlink(cfg)
end

local netcfg_template = DC.OBJ({
	enable_mdns = DC.BOOL(false),

	-- A set of ethernet interfaces configures
	ethernets = DC.SET( DC.OBJ{
		enable = DC.BOOL(true),
		method = DC.STR("dhcp"),
		address = DC.STR(""),
		netmask = DC.STR("255.255.255.0"),
		gw = DC.STR(""),
		mac = DC.STR(""),
		dns = DC.ARR( DC.STR("") ),
		dhcp_server = DC.OBJ{
			enable = DC.BOOL(false),
			dynamic_start = DC.INT(100),
			dynamic_end = DC.INT(200),
			gw = DC.STR("self"),
			dns = DC.STR("auto")
		},
	} : __changed( on_etherChanged )
	),

	--Network aliases
	aliases = DC.SET( DC.OBJ{
		enable = DC.BOOL(true),
		address = DC.STR(""),
		netmask = DC.STR("255.255.255.0"),
		gw = DC.STR(""),
		address2 = DC.STR(""),
		netmask2 = DC.STR("255.255.255.0"),
		gw2 = DC.STR("")
	} : __changed( on_aliasChanged )
	),

	--A internal link, for some special purpose.
	interlink = DC.OBJ {
		ifname = DC.STR("usb0"),
		enable = DC.BOOL(true),
		address = DC.STR( "10.254.254.1" ),
		netmask = DC.STR( "255.255.255.0" )
	} : __changed( on_interlinkChanged )

	--TODO: Others.
}) : __changed( function(conf)
	conf:TAB_SAVE()
end)

-----------------------------------------------------------

function _M.Load(failDefaultCfg)
	local conf = CONF.Create({}, netcfg_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r and failDefaultCfg then --Try load a fail-default configure file, to configure default network.
		conf:TAB_STORAGE( "file", { file = failDefaultCfg } )
		conf:TAB_LOAD()
		conf:TAB_STORAGE( "file", { file = _M.config_file } ) -- Bind back to configure file.
	end

	conf:TAB_RESET_CHANGE()
	return conf
end

-----------------------------------------------------------

return _M


