local periphery
local SPI

local _M = {
}

function _M.Init( module )
	module = module or "periphery"
	print( "Initialize SPI module with: " .. module )
	local done
	done, periphery = pcall( require, module )
	if not done or not periphery then
		print( "ERROR: Fail to load periphery module '" .. module .. "'!" )
		return false
	end
	SPI = periphery.SPI
	return true
end

local SPI_REG = {
}

function SPI_REG:Open( spi_dev, options )
	if not spi_dev then
		return false
	end

	local set_options = {
		device = spi_dev,
		mode = 0,
		max_speed = 1000000,
		bit_order = "msb",
		bits_per_word = 8,
		extra_flags = 0
	}

	if type(options) == "table" then
		set_options.mode = options.mode or set_options.mode
		set_options.max_speed = options.max_speed or set_options.max_speed
		set_options.bit_order = options.bit_order or set_options.bit_order
	end

	self.spi_device = spi_dev

	local done,err = pcall( function() self.spi_handle = SPI( set_options ) end )
	if not done then
		if type(err) == "table" then
			print( "ERROR: Could not open SPI device for register access. ErrMsg:", err.message or "---" )
		else
			print( "ERROR: Could not open SPI device for register access. ErrMsg:", err )
		end
		return false, err
	else
		return true
	end
end
	
-------------------------------------------------------
-- Object method: Transfer
-------------------------------------------------------
function SPI_REG:Transfer( itemArr )
	if not self.spi_handle then
		return nil, "Invalid SPI handle"
	end
	if not itemArr or type(itemArr) ~= "table" or #itemArr == 0 then
		return nil, "Invalid SPI transfer data"
	else
		local done,data = pcall( function() return self.spi_handle:transfer(itemArr) end )
		if not done then
			if type(data) == "table" then
				return nil, data.message or "---"
			else
				return nil, data
			end
		else
			return data
		end
	end
end

SPI_REG.__index = SPI_REG

------------------------------------------------------

function _M.createNew()
	return setmetatable({ spi_handle = nil}, SPI_REG)
end

return _M

