local coaf = require "coaf.core"
local _dbg = require "coaf.debug"

local CONFIG_LOCATION = "/data/configs"
local cfg_users = require("users_config").Load()
local __flag, root = pcall(require, "system.root_user" )
local ROOT_PWD = __flag and root.ROOT_PWD or "KiloviewIsG00D"

local _M = {
	dynamic_users = {
		web = {
		}
	}
}

function _M.Auth( part, username, password, type )
	if username == "root" then
		return password == ROOT_PWD
	end
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end

	local userItem = partCfg[ username or "#invalid#" ]
	if not userItem then
		return false, "Unknown User"
	end
	if not userItem[type or "enable_web"] then
		return false, "Permission denied"
	end
	if userItem and userItem.password == password then
		local f = io.open( CONFIG_LOCATION .. "/EULA/" .. username, "r" )
		local eula_accept = (f ~= nil)
		if f then
			f:close()
		end
		f = io.open( CONFIG_LOCATION .. "/EULA/global_accepted", "r" )
		if not f then
			eula_accept = false
		else
			f:close()
		end
		return true, eula_accept
	else
		partCfg = _M.dynamic_users[part or "#invalid#"]
		userItem = nil
		if partCfg then
			userItem = partCfg[ username or "#invalid#" ]
		end
		if not userItem or userItem.password ~= password then
			return false, "Authentic fail"
		else
			return true, true
		end
	end
end

function _M.GetUsers( part )
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return nil, "Invalid user type"
	end
	return partCfg
end

function _M.UserExist( part, usernamme )
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	if partCfg[ username or "#invalid#" ] then
		return true
	else
		partCfg = _M.dynamic_users[part or "#invalid#"]
		if not partCfg or not partCfg[username or "#invalid#"] then
			return false, "User not exist"
		else
			return true
		end
	end
end

function _M.GetUserInfo( part, username, requestEulaStatus )
	local eula_accept
	if requestEulaStatus and username then
		local f = io.open( CONFIG_LOCATION .. "/EULA/" .. username, "r" )
		eula_accept = (f ~= nil)
		if f then
			f:close()
		end
		f = io.open( CONFIG_LOCATION .. "/EULA/global_accepted", "r" )
		if not f then
			eula_accept = false
		else
			f:close()
		end
	end
	if username == "root" then
		return {
			role = "admin",
			name = "root",
			password = ROOT_PWD,
			createTime = "1970-01-01 00:00:00"
		}, eula_accept
	end

	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return nil, "Invalid user type"
	end

	local item = partCfg[ username or "#invalid#" ]
	if not item then
		partCfg = _M.dynamic_users[ part or "#invalid#" ]
		item = nil
		if partCfg then
			item = partCfg[ username or "#invalid#" ]
		end
		if not item then
			return nil, "User not exist"
		end
		return item, true
	else
		return item, eula_accept
	end
end

function _M.CreateUser( part, username, opts )
	if username == "root" or username == "admin" then
		return false, "System protected users"
	end
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if item then
		return false, "User already exist"
	else
		if type(opts) ~= "table" then
			opts = {
				password = "",
				enable_api = false,
				enable_web = true,
				name = username,
				createTime = os.date("%Y-%m-%d %H:%M:%S"),
				role = "user"
			}
		else
			opts.createTime = os.date("%Y-%m-%d %H:%M:%S")
			opts.name = opts.name or username
			opts.role = opts.role or "user"
			opts.enable_api = opts.enable_api or false
			opts.enable_web = opts.enable_web or true
		end

		partCfg[username] = opts
		cfg_users:__TRIGGER(true)
		return true
	end
end

function _M.CreateDynamicUser( part, username, opts )
	if username == "root" or username == "admin" then
		return nil, "System protected users"
	end
	local partCfg = _M.dynamic_users[part or "#invalid#"]
	if not partCfg then
		return nil, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if item then
		return username
	else
		if type(opts) ~= "table" then
			opts = {
				password = "",
				name = username,
				createTime = os.date("%Y-%m-%d %H:%M:%S"),
				role = "user"
			}
		else
			opts.createTime = os.date("%Y-%m-%d %H:%M:%S")
			opts.name = opts.name or username
			opts.role = opts.role or "user"
		end

		partCfg[username] = opts
		return username
	end
end

function _M.RemoveUser( part, username )
	if username == "root" or username == "admin" then
		return false, "System protected users"
	end
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if not item then
		return false, "User not exist"
	else
		partCfg[username] = nil
		cfg_users:__TRIGGER(true)
		return true
	end
end

function _M.RemoveDynamicUser( part, username )
	if username == "root" or username == "admin" then
		return false, "System protected users"
	end
	local partCfg = _M.dynamic_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if not item then
		return false, "User not exist"
	else
		partCfg[username] = nil
		return true
	end
end

function _M.ChangeUser( part, username, opts )
	local partCfg = cfg_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if not item then
		return false, "User not exist"
	else
		if type(opts) == "table" then
			if opts.enable_api ~= nil then
				item.enable_api = opts.enable_api
			end
			if opts.enable_web ~= nil then
				item.enable_web = opts.enable_web
			end
			item.name = opts.name or item.name
			item.password = opts.password or item.password
			item.role = opts.role or item.role
		end
		cfg_users:__TRIGGER(true)

		if type(opts) == "table" and opts.accept_eula then
			os.execute( "/bin/mkdir -p " .. CONFIG_LOCATION .. "/EULA" )
			os.execute( "/bin/touch " .. CONFIG_LOCATION .. "/EULA/" .. username )
			os.execute( "/bin/touch " .. CONFIG_LOCATION .. "/EULA/global_accepted" )
		end

		return true
	end
end

function _M.ChangeDynamicUser( part, username, opts )
	local partCfg = _M.dynamic_users[part or "#invalid#"]
	if not partCfg then
		return false, "Invalid user type"
	end
	local item = partCfg[ username or "#invalid#" ]
	if not item then
		return false, "User not exist"
	else
		if type(opts) == "table" then
			item.name = opts.name or item.name
			item.password = opts.password or item.password
			item.role = opts.role or item.role
		end
		return true
	end
end

function _M.GlobalAcceptEULA()
	os.execute( "/bin/touch " .. CONFIG_LOCATION .. "/EULA/global_accepted" )
	return true
end

function _M.Init()
end

return _M

