local DC = require "supertable.declare"
local CONF = require "supertable.core"

----------------------------
local _M = {
	config_location = "/data/configs",
	config_file = "users.conf"
}

----------------------------

local users_template = DC.OBJ({
	web = DC.SET(
		DC.OBJ({
			password = DC.STR(""),
			name = DC.STR(""),
			createTime = DC.STR(""),
			enable_api = DC.BOOL(false),
			enable_web = DC.BOOL(true),
			role = DC.STR("user")
		})
	)
}) : __changed( function(conf)
	conf:TAB_SAVE()
end)

----------------------------

local failDefault = {
	web = {
		["admin"] = { password = "admin", name = "Admin", createTime = "1970-01-01 00:00:00", role = "admin", enable_api = false, enable_web = true }
	}
}

-------------------------------------------------------

function _M.Load()
	local conf = CONF.Create({}, users_template)
	conf:TAB_STORAGE( "file", { file =  _M.config_location .. "/" .. _M.config_file } )
	local _, r = conf:TAB_LOAD()
	if not r then --Try load a fail-default configure 
		conf:TAB_BIND(failDefault)
	end
	conf:TAB_RESET_CHANGE()
	return conf
end

-------------------------------------------------------

return _M


