--local coaf = require "coaf.core"
local _dbg = require "coaf.debug"
local protocol = require "coaf.rpc.protocol"

--------------------------------------
local _M = {
	max_packet_size = 4096,
	created_streamers = {
	}
}

--------------------------------------

local streamer_meta = {}

function streamer_meta:__PutData(data)
	if self.connected then
		if type(data) == "table" then
			local i
			for i=1,#data do
				table.insert( self.data, {
					data = data[i],
					size = #data[i],
					read_pos = 1 
				})
			end
		else
			table.insert( self.data, {
				data = data,
				size = #data,
				read_pos = 1 
			})
		end
	end
end

function streamer_meta:Read( size_pattern )
	if not self.connected then
		return nil, "closed", 0
	end

	local ret

	local size = tonumber(size_pattern)
	if size then
		if size > 0 then
			local i,item
			local actual_size = 0
			local this_size
			local drop_to
			for i,item in ipairs(self.data) do
				if item.read_pos <= item.size then
					this_size = (item.size - item.read_pos) + 1
					if this_size > size then
						this_size = size
						if not ret then
							ret = { item.data:sub( item.read_pos, item.read_pos + this_size - 1 ) }
						else
							table.insert( ret, { item.data:sub( item.read_pos, item.read_pos + this_size - 1) } )
						end
						item.read_pos = item.read_pos + this_size
					elseif item.read_pos == 1 then
						if not ret then
							ret = { item.data }
						else
							table.insert( ret, item.data )
						end
						drop_to = i
					else
						if not ret then
							ret = { item.data:sub( item.read_pos ) }
						else
							table.insert( ret, item.data:sub( item.read_pos ) )
						end
						drop_to = i
					end
					actual_size = actual_size + this_size
					size = size - this_size
					if size == 0 then
						break
					end
				else
					drop_to = i
				end
			end

			if drop_to then
				for i=1, drop_to do
					table.remove( self.data, 1 )
				end
				collectgarbage()
			end

			if not ret then
				return nil, "partial", 0
			end

			if actual_size >= size then
				return table.concat(ret), nil, actual_size
			else
				return table.concat(ret), "partial", actual_size
			end
		else
			return nil, "invalid", 0
		end
	elseif not size_pattern or size_pattern == "*a" or size_pattern == "a" or size_pattern == "*" or size_pattern == "" then
		local i,item
		local actual_size = 0
		local this_size
		for i,item in ipairs(self.data) do
			if item.read_pos <= item.size then
				this_size = (item.size - item.read_pos) + 1
				if item.read_pos == 1 then
					if not ret then
						ret = { item.data }
					else
						table.insert( ret, item.data )
					end
				else
					if not ret then
						ret = { item.data:sub( item.read_pos ) }
					else
						table.insert( ret, item.data:sub( item.read_pos ) )
					end
				end
				actual_size = actual_size + this_size
			end
		end
		if not ret then
			return nil, "partial", 0
		end
		self.data = {}
		collectgarbage()
		return table.concat(ret), nil, actual_size
	elseif size_pattern == "*l" or size_pattern == "l" then
		local i,item
		local actual_size = 0
		local drop_to, ensured
		local token,rem
		local this_size
		for i,item in ipairs(self.data) do
			if item.read_pos <= item.size then
				this_size = (item.size - item.read_pos) + 1
				local token, rem
				if item.read_pos == 1 then
					token,rem = item.data:match("^(.-)[\r\n]+(.*)$")
				else
					token,rem = item.data:sub(item.read_pos):match("^(.-)[\r\n]+(.*)$")
				end

				if token then
					this_size = #token
					if not ret then
						ret = { token }
					else
						table.insert( ret, token )
					end
					if rem == "" then
						drop_to = i
					else
						drop_to = i-1
						item.data = rem
						item.size = #rem
						item.read_pos = 1
					end
					actual_size = actual_size + this_size
					ensured = true
					break
				elseif item.read_pos == 1 then
					if not ret then
						ret = { item.data }
					else
						table.insert( ret, item.data )
					end
				else
					if not ret then
						ret = { item.data:sub(item.read_pos) }
					else
						table.insert( ret, item.data:sub(item.read_pos) )
					end
				end
				actual_size = actual_size + this_size
			else
				drop_to = i
			end
		end

		if drop_to then
			for i=1, drop_to do
				table.remove( self.data, 1 )
			end
			collectgarbage()
		end

		if not ensured then
			return nil, "partial", 0
		end

		return table.concat(ret), nil, actual_size
	else
		return nil, "invalid", 0
	end
end

function streamer_meta:Write( data, i, j )
	if not self.connected or not self.related_connection then
		return -1, "closed"
	end
	if not data then
		return -1, "invalid"
	end

	i = i or 1
	j = j or #data

	local sent = 0
	local pkt, pkt_size
	local cc, flag
	while i <= j do
		pkt_size = j-i+1
		if pkt_size > _M.max_packet_size then
			pkt_size = _M.max_packet_size
		end

		pkt = data:sub(i, i+pkt_size-1)
		sent = sent + pkt_size
		i = i + pkt_size
		local ack = {
			SN = 0, --self.LAST_SN,
			CMD = self.CMD,
			ARGS = "content",
			headers = {
				SVC = self.SVC,
				OBJ = self.OBJ,
				LOC = self.LOC,
				VIA = self.VIA,
				RESULT = "OK"
			},
			data = pkt
		}
		cc,flag = self.related_connection:Send( table.concat( protocol.FormatContent(ack) ) )
		if not cc then
			return nil, flag
		end
	end
	return sent, nil
end

function streamer_meta:Disconnect()
	if self.connected and self.related_connection then
		local ack = {
			SN = self.LAST_SN,
			CMD = self.CMD,
			ARGS = "disconnect",
			headers = {
				SVC = self.SVC,
				OBJ = self.OBJ,
				LOC = self.LOC,
				VIA = self.VIA,
				RESULT = "OK"
			}
		}
		self.related_connection:Send( table.concat( protocol.FormatContent(ack) ) )

		self.related_connection = nil
	end
	self.connected = false
end

function streamer_meta:Destroy(disconn)
	if disconn then
		self:Disconnect()
	end
	self.connected = false
	self.related_connection = nil
	_M.created_streamers[ self.my_idr ] = nil
	collectgarbage()
end

streamer_meta.__index = streamer_meta

--------------------------------------

function _M.CreateStreamer( source_idr, connection )
	if not source_idr or not connection then
		return nil, "Invalid source ID or connection"
	end

	local idr = source_idr .. "|" .. tostring(connection)
	if _M.created_streamers[idr] then
		return _M.created_streamers[idr], "old"
	end

	local streamer = {
		CMD = "ACK/STREAM",
		OBJ = nil,
		VIA = nil,
		SVC = nil,
		LOC = nil,
		LAST_SN = 0,

		data = {},

		connected = true,
		my_idr = idr,
		source_idr = source_idr,
		related_connection = connection
	}

	setmetatable( streamer, streamer_meta )
	_M.created_streamers[idr] = streamer
	return streamer, "new"
end

function _M.DestoryStreamer( source_idr, connection, disconn )
	local k,v
	local removes = {}
	for k,v in pairs(_M.created_streamers) do
		if (not source_idr or source_idr == v.source_idr) and
			(not connection or connection == v.connection ) then
			table.insert(removes,v)
			if source_idr and connection then
				break
			end
		end
	end

	for k,v in pairs(removes) do
		v:Destroy(disconn)
	end
end

return _M

