#!/bin/sh

export PROD_ID="N4"
export TARGET_TMP_DIR="/upgrade/.upgrade_tmp"

export SWVER="1.60.0160"
export LUA_VERSION="5.3"

export ROOT_REQ_KB="-7500"
export DATA_REQ_KB="3944"
export UPGRADE_RETURNS=1

#!/bin/sh
export PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
mkdir -p /data/update_log
mkdir -p $TARGET_TMP_DIR/_UPGRADE

updatetime=`date "+%Y-%m-%d %H:%M:%S"`
updatetimecode=`date "+%Y%m%d%H%M"`
WITH_WARNING=0
WITH_ERROR=0
logfile="/data/update_log/$SWVER-$updatetimecode.log"

echo "Upgrade to Software version $SWVER..." > $logfile
echo "Upgrade time: $updatetime" >> $logfile

#NOTE: Must disable watch-dog while upgrading!
if [ -x /sbin/wdog ]; then
	/sbin/wdog disable
	echo "Disabled watch-dog while upgrading." >> $logfile
fi

echo "------------------------------------" >> $logfile

if [ $# -gt 0 ]; then
    cd $1
else
    cd /upgrade
fi

tar xzf ./patch.tar.gz -C $TARGET_TMP_DIR/_UPGRADE 
if [ $? -ne 0 ]; then
	echo "Upgrading fail! Error to extract the patch file!" >> $logfile
	rm -Rf $TARGET_TMP_DIR
	exit 1
fi

rm -Rf ./patch.tar.gz
cd $TARGET_TMP_DIR	#Change to target position

getDiskFree()
{
	#Because the K2E busybox system only support human format
	local freeSpaces=`df -h $1 | sed 1d | awk '{print $4}'`
	if [ "${freeSpaces/G/}" != "${freeSpaces}" ]; then
		#Is GB
		freeSpaces=${freeSpaces/G/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024 * 1024))"`
	elif [ "${freeSpaces/M/}" != "${freeSpaces}" ]; then
		#Is MB
		freeSpaces=${freeSpaces/M/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces * 1024))"`
	elif [ "${freeSpaces/K/}" != "${freeSpaces}" ]; then
		#Is KB
		freeSpaces=${freeSpaces/K/}
		freeSpaces=`lua -e "print(math.floor($freeSpaces))"`
	fi
	echo $freeSpaces
	exit 0
}

root_free_sz=`getDiskFree /`
data_free_sz=`getDiskFree /data`

root_free_sz=`expr $root_free_sz - 1024`	#Keep 1MB at least
data_free_sz=`expr $data_free_sz - 1024`	#Keep 1MB at least

echo "ROOT: Require $ROOT_REQ_KB KB, valid $root_free_sz KB." >> $logfile
echo "DATA: Require $DATA_REQ_KB KB, valid $data_free_sz KB." >> $logfile

if [ $root_free_sz -lt $ROOT_REQ_KB -o $data_free_sz -lt $DATA_REQ_KB ]; then
	echo "ERROR: Insufficent disk spaces" >> $logfile
	exit 1
fi

upgrade()
{
	if [ -d "$TARGET_TMP_DIR/_UPGRADE/fakeroot" ]; then
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/fakeroot/"* / >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/datafs" ]; then
		if [ ! -d /data ]; then
			mkdir /data
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/datafs/"* /data/ >> $logfile 2>&1
	fi

	if [ -d "$TARGET_TMP_DIR/_UPGRADE/boot" ]; then
		this_part_id="OTHER"
		default_boot_for_part_id="OTHER"

		if [ -f /boot/PART_ID ]; then
			this_part_id=`cat /boot/PART_ID`
		fi
		if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" ]; then
			default_boot_for_part_id=`cat $TARGET_TMP_DIR/_UPGRADE/boot/boot.def`
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/boot.def" 
		fi

		#Don't touch PART_ID, if exists.
		#rm "$TARGET_TMP_DIR/_UPGRADE/boot/PART_ID"

		#2 cases: 1) part_id matched; 2) both part_id and 'boot.def' are empty ("OTHER" identified).
		if [ "$default_boot_for_part_id" = "$this_part_id" ]; then
			#Keep default BOOT.BIN, drop others.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
		elif [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" ]; then
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			mv "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT-$this_part_id.BIN" "$TARGET_TMP_DIR/_UPGRADE/boot/BOOT.BIN"
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT-*.BIN

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/enc_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit-*.bin
			fi

			if [ -f "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" ]; then
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit.bin
				mv "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit-$this_part_id.bin" "$TARGET_TMP_DIR/_UPGRADE/boot/dec_bit.bin"
				rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit-*.bin
			fi
		else
			#Don't touch BOOT.BIN and other .bit files.
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"BOOT*.BIN
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"enc_bit*.bin
			rm "$TARGET_TMP_DIR/_UPGRADE/boot/"dec_bit*.bin
		fi
		cp -Rf "$TARGET_TMP_DIR/_UPGRADE/boot/"* /boot/ >> $logfile 2>&1
	fi

	return 0
}

drop()
{
	if [ "$1" = "fakeroot" ]; then
		if [ -e "$2" ]; then
			rm -Rf "$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	elif [ "$1" = "datafs" ]; then
		if [ -e "/data$2" ]; then
			rm -Rf "/data$2"
			if [ $? -ne 0 ]; then
			    echo "WARNING: FAIL to drop un-used item [/data$2]." >> $logfile
			    WITH_WARNING=`expr $WITH_WARNING + 1`
			    return 1
			fi
		fi
	fi
    return 0
}

## The upgrading content below: (generated by release tool)

drop datafs "/web/html/static/css/app.38079361f6c3b1b78f465ec5e732c658.css"
drop datafs "/web/html/static/css/app.38079361f6c3b1b78f465ec5e732c658.css.map"
drop datafs "/web/html/static/css/app.403344fb9931fe337946b6adf853d536.css"
drop datafs "/web/html/static/css/app.48143648be2e7c41e8030d03b2b8eea4.css"
drop datafs "/web/html/static/css/app.48143648be2e7c41e8030d03b2b8eea4.css.map"
drop datafs "/web/html/static/css/app.4b4bf550555502b0e3c2c319dbb25467.css"
drop datafs "/web/html/static/css/app.52e1801870c708e9c6a22f43126ed5b7.css"
drop datafs "/web/html/static/css/app.52e1801870c708e9c6a22f43126ed5b7.css.map"
drop datafs "/web/html/static/css/app.55d73c5a12f0e229b0d6416cc0a02a15.css"
drop datafs "/web/html/static/css/app.a289dabdc0b4189016ef9a0d3c641296.css"
drop datafs "/web/html/static/css/app.a393a13ad1cbde3c2f613aa565620df3.css"
drop datafs "/web/html/static/css/app.a428b37267c28b0faa0e9e81aa2dcb6f.css"
drop datafs "/web/html/static/css/app.a78e073e4363d91b51b25d3a5329505b.css"
drop datafs "/web/html/static/css/app.ab4fe621574a922413a9579cd38f1d46.css"
drop datafs "/web/html/static/css/app.b39fa1a5ad16b0412e4fb4ef8ba06226.css"
drop datafs "/web/html/static/css/app.b39fa1a5ad16b0412e4fb4ef8ba06226.css.map"
drop datafs "/web/html/static/css/app.c5067c67db82a5e777a9a5b708d36eca.css"
drop datafs "/web/html/static/css/app.cd22041bbeb2b113ea19f72d624223cb.css"
drop datafs "/web/html/static/css/app.d2b4465ae79a32055766df19abd242cb.css"
drop datafs "/web/html/static/css/app.d682999283999fbbb38de1ec0c552c89.css"
drop datafs "/web/html/static/css/app.d682999283999fbbb38de1ec0c552c89.css.map"
drop datafs "/web/html/static/css/app.d8e2717647fd7e44c8cd85e9f20b66db.css"
drop datafs "/web/html/static/css/app.eeb9f31150c9260361cb3bb6b1f239e7.css"
drop datafs "/web/html/static/css/app.f66060ae784798d1bf7a1203f113b370.css"
drop datafs "/web/html/static/css/app.f66060ae784798d1bf7a1203f113b370.css.map"
drop datafs "/web/html/static/fonts/element-icons.6f0a763.ttf"
drop datafs "/web/html/static/fonts/iconfont.4c0cf5b.eot"
drop datafs "/web/html/static/fonts/iconfont.e45de52.ttf"
drop datafs "/web/html/static/img/iconfont.1760417.svg"
drop datafs "/web/html/static/img/iconfont.a5ae94a.svg"
drop datafs "/web/html/static/img/logo.0bef24a.png"
drop datafs "/web/html/static/img/logo.*.png"
drop datafs "/web/html/static/js/0.4584fa3277c7ae3943ac.js"
drop datafs "/web/html/static/js/0.c0e45efd184603d39a15.js"
drop datafs "/web/html/static/js/0.c0e45efd184603d39a15.js.map"
drop datafs "/web/html/static/js/1.3b1ba1b8208bda813ba0.js"
drop datafs "/web/html/static/js/1.3b1ba1b8208bda813ba0.js.map"
drop datafs "/web/html/static/js/1.41d00789619c7239a85f.js"
drop datafs "/web/html/static/js/app.038c38b3987c6aab8a73.js"
drop datafs "/web/html/static/js/app.0dcc49cfd7f71a619415.js"
drop datafs "/web/html/static/js/app.2aa1f9bcc84549f165ba.js"
drop datafs "/web/html/static/js/app.2f0af0b32d7f0041b5ee.js"
drop datafs "/web/html/static/js/app.2f0af0b32d7f0041b5ee.js.map"
drop datafs "/web/html/static/js/app.43a83f5c11b9e9a55814.js"
drop datafs "/web/html/static/js/app.47b2a5d5d7470532b5fe.js"
drop datafs "/web/html/static/js/app.58823bda59f9a2d4be39.js"
drop datafs "/web/html/static/js/app.603f8f7a88681370834a.js"
drop datafs "/web/html/static/js/app.6c5640867f167d8a7903.js"
drop datafs "/web/html/static/js/app.6d5c9a1f5d523daa3c00.js"
drop datafs "/web/html/static/js/app.81cd8d451738da5e131e.js"
drop datafs "/web/html/static/js/app.8aa5575d76d66cef874a.js"
drop datafs "/web/html/static/js/app.9abc90893ab92f24fee7.js"
drop datafs "/web/html/static/js/app.a086bafee1d69ad73b40.js"
drop datafs "/web/html/static/js/app.c76e02c14883e2547a19.js"
drop datafs "/web/html/static/js/app.e73ecf0fc2b0e3ac0833.js"
drop datafs "/web/html/static/js/app.f55e2a9f70416906bff4.js"
drop datafs "/web/html/static/js/decoderMode.01e1359ccd118c1fed3d.js"
drop datafs "/web/html/static/js/decoderMode.044217fc4280a976491c.js"
drop datafs "/web/html/static/js/decoderMode.0af4b38727e6c938ffe1.js"
drop datafs "/web/html/static/js/decoderMode.6231cedeec00a880b2ad.js"
drop datafs "/web/html/static/js/decoderMode.732370b1d691f3a24582.js"
drop datafs "/web/html/static/js/decoderMode.976af6d8c6c28642089a.js"
drop datafs "/web/html/static/js/decoderMode.e611d97a09076de7242f.js"
drop datafs "/web/html/static/js/decoderMode.fddc513be8ad61541aba.js"
drop datafs "/web/html/static/js/encoderMode.40117591a8bf93fb8448.js"
drop datafs "/web/html/static/js/encoderMode.59c5a02067f5966aa6d2.js"
drop datafs "/web/html/static/js/encoderMode.7e13da2b0d5e8e58d17d.js"
drop datafs "/web/html/static/js/encoderMode.a45d1cfb5583f1259629.js"
drop datafs "/web/html/static/js/encoderMode.b2025aac19934eb88221.js"
drop datafs "/web/html/static/js/encoderMode.d3d9a97b94871ed22222.js"
drop datafs "/web/html/static/js/firmwareudp.d1abe279ddcf8b1417c1.js"
drop datafs "/web/html/static/js/firmwareudp.deec2552a51ce8502d39.js"
drop datafs "/web/html/static/js/home.1c6985293d16903dec6e.js"
drop datafs "/web/html/static/js/home.5cb63ba751e0badca11d.js"
drop datafs "/web/html/static/js/home.74b81a7f366acba3b1c5.js"
drop datafs "/web/html/static/js/home.c9095dfe8c52850bcd90.js"
drop datafs "/web/html/static/js/home.ebc7e22abe27865c8741.js"
drop datafs "/web/html/static/js/home.ec3295596571279a30e2.js"
drop datafs "/web/html/static/js/manifest.18131ddd33f8522033a1.js"
drop datafs "/web/html/static/js/manifest.29e2da3276ac4cf3e9f2.js"
drop datafs "/web/html/static/js/manifest.2f8e6f5c3719537062c0.js"
drop datafs "/web/html/static/js/manifest.4041ed1671345182c67e.js"
drop datafs "/web/html/static/js/manifest.4041ed1671345182c67e.js.map"
drop datafs "/web/html/static/js/manifest.674bf4162335d4fa51b1.js"
drop datafs "/web/html/static/js/manifest.697da8bf906a1a862f68.js"
drop datafs "/web/html/static/js/manifest.71150de2062932e64473.js"
drop datafs "/web/html/static/js/manifest.8281b1b12e50db8f51e3.js"
drop datafs "/web/html/static/js/manifest.9f98a2dcabb444fb108f.js"
drop datafs "/web/html/static/js/manifest.ae1a46d63a0e9f8b14d1.js"
drop datafs "/web/html/static/js/manifest.b569c1f01934deff4a35.js"
drop datafs "/web/html/static/js/manifest.b702f3039687518642b5.js"
drop datafs "/web/html/static/js/manifest.c8f7b2007719a8a108b3.js"
drop datafs "/web/html/static/js/manifest.e1a41d41c0894cb399a2.js"
drop datafs "/web/html/static/js/manifest.ea7bbea836349cd67103.js"
drop datafs "/web/html/static/js/networksetup.399ed63084cd41d5f193.js"
drop datafs "/web/html/static/js/networksetup.92b4b8a62b3af840e84b.js"
drop datafs "/web/html/static/js/softwareProtocol.964ebf68d9c71998b9d9.js"
drop datafs "/web/html/static/js/softwareProtocol.f67e853e2b9c343f7406.js"
drop datafs "/web/html/static/js/superMode.016d00d0f394ab2fdae6.js"
drop datafs "/web/html/static/js/system-time.6a27ca09e27730f88be6.js"
drop datafs "/web/html/static/js/users.22f45e9168c0bc2b15fe.js"
drop datafs "/web/html/static/js/users.f999d4c67dc1e954dcf0.js"
drop datafs "/web/html/static/js/vendor.161f9f7535623ba13d16.js"
drop datafs "/web/html/static/js/vendor.c44d7ca056c52c0e9575.js"
drop datafs "/web/html/static/js/vendor.c44d7ca056c52c0e9575.js.map"
drop datafs "/web/html/static/js/vendor.d18b4a2ecef6ffe3f07d.js"
drop datafs "/web/html/static/mock/login.json"
drop datafs "/web/html/static/mock/software.json"
drop fakeroot "/.stage.apt/etc/apt/apt.conf.d/apt.conf"
drop fakeroot "/.stage.apt/etc/apt/preferences.d/preferences"
drop fakeroot "/.stage.apt/etc/apt/sources.list"
drop fakeroot "/.stage.apt/usr/bin/update-alternatives"
drop fakeroot "/.stage.apt/usr/sbin/update-rc.d"
drop fakeroot "/.stage.apt/var/cache/apt/archives/lock"
drop fakeroot "/.stage.apt/var/lib/apt/extended_states"
drop fakeroot "/.stage.apt/var/lib/apt/lists/lock"
drop fakeroot "/.stage.apt/var/lib/apt/lists/_opt_Xilinx_petalinux-v2015.4-final_components_packages-repo_plnx-repo_dists_default_main_binary-cortexa9-vfp-neon_Packages"
drop fakeroot "/.stage.apt/var/log/apt/history.log"
drop fakeroot "/.stage.apt/var/log/apt/term.log"
drop fakeroot "/.targetroot.apt/etc/apt/apt.conf.d/apt.conf"
drop fakeroot "/.targetroot.apt/etc/apt/preferences.d/preferences"
drop fakeroot "/.targetroot.apt/etc/apt/sources.list"
drop fakeroot "/.targetroot.apt/usr/bin/update-alternatives"
drop fakeroot "/.targetroot.apt/usr/sbin/update-rc.d"
drop fakeroot "/.targetroot.apt/var/cache/apt/archives/lock"
drop fakeroot "/.targetroot.apt/var/lib/apt/extended_states"
drop fakeroot "/.targetroot.apt/var/lib/apt/lists/lock"
drop fakeroot "/.targetroot.apt/var/lib/apt/lists/_opt_Xilinx_petalinux-v2015.4-final_components_packages-repo_plnx-repo_dists_default_main_binary-cortexa9-vfp-neon_Packages"
drop fakeroot "/.targetroot.apt/var/log/apt/history.log"
drop fakeroot "/.targetroot.apt/var/log/apt/term.log"
drop fakeroot "/usr/bin/rtwpriv"
drop fakeroot "/usr/lib/libndi.so.4.0.0"
drop fakeroot "/usr/lib/libndi.so.4.1.0"
drop fakeroot "/usr/lib/libndi.so.4.1.4"
drop fakeroot "/usr/lib/libndi.so.4.2.0"
drop fakeroot "/usr/lib/libndi.so.4.5.0"
drop fakeroot "/usr/lib/lua/5.3/OnvifSvc.so"
drop fakeroot "/usr/lib/lua/5.3/openssl.so"
drop fakeroot "/usr/local/bin/codec/chips/ite6661.lua"
drop fakeroot "/usr/local/bin/codec/hdmi_chips.lua"
drop fakeroot "/usr/local/bin/codec/iocfg.lua"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/lauxlib.h"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/luaconf.h"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/lua.h"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/lua.hpp"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/luajit.h"
drop fakeroot "/usr/local/openresty/luajit/include/luajit-2.1/lualib.h"
drop fakeroot "/usr/local/openresty/luajit/share/man/man1/luajit.1"
drop fakeroot "/usr/local/openresty/pod/array-var-nginx-module-0.05/array-var-nginx-module-0.05.pod"
drop fakeroot "/usr/local/openresty/pod/drizzle-nginx-module-0.1.10/drizzle-nginx-module-0.1.10.pod"
drop fakeroot "/usr/local/openresty/pod/echo-nginx-module-0.60/echo-nginx-module-0.60.pod"
drop fakeroot "/usr/local/openresty/pod/encrypted-session-nginx-module-0.06/encrypted-session-nginx-module-0.06.pod"
drop fakeroot "/usr/local/openresty/pod/form-input-nginx-module-0.12/form-input-nginx-module-0.12.pod"
drop fakeroot "/usr/local/openresty/pod/headers-more-nginx-module-0.32/headers-more-nginx-module-0.32.pod"
drop fakeroot "/usr/local/openresty/pod/iconv-nginx-module-0.14/iconv-nginx-module-0.14.pod"
drop fakeroot "/usr/local/openresty/pod/lua-5.1.5/lua-5.1.5.pod"
drop fakeroot "/usr/local/openresty/pod/lua-cjson-2.1.0.5/lua-cjson-2.1.0.5.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/changes.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/contact.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_c_api.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/extensions.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_ffi_api.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_ffi.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_ffi_semantics.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_ffi_tutorial.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_jit.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/ext_profiler.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/faq.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/install.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/luajit-2.1.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/running.pod"
drop fakeroot "/usr/local/openresty/pod/luajit-2.1/status.pod"
drop fakeroot "/usr/local/openresty/pod/lua-redis-parser-0.13/lua-redis-parser-0.13.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/lua-resty-core-0.1.11.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.balancer.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.ocsp.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.re.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.semaphore.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.ssl.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-core-0.1.11/ngx.ssl.session.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-dns-0.18/lua-resty-dns-0.18.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-limit-traffic-0.03/lua-resty-limit-traffic-0.03.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-limit-traffic-0.03/resty.limit.conn.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-limit-traffic-0.03/resty.limit.req.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-limit-traffic-0.03/resty.limit.traffic.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-lock-0.06/lua-resty-lock-0.06.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-lrucache-0.06/lua-resty-lrucache-0.06.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-memcached-0.14/lua-resty-memcached-0.14.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-mysql-0.19/.github.issue_template.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-mysql-0.19/lua-resty-mysql-0.19.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-redis-0.26/lua-resty-redis-0.26.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-string-0.09/lua-resty-string-0.09.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-upload-0.10/lua-resty-upload-0.10.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-upstream-healthcheck-0.04/lua-resty-upstream-healthcheck-0.04.pod"
drop fakeroot "/usr/local/openresty/pod/lua-resty-websocket-0.06/lua-resty-websocket-0.06.pod"
drop fakeroot "/usr/local/openresty/pod/memc-nginx-module-0.18/memc-nginx-module-0.18.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/accept_failed.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/beginners_guide.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/chunked_encoding_from_backend.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/configure.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/configuring_https_servers.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/contributing_changes.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/control.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/converting_rewrite_rules.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/daemon_master_process_off.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/debugging_log.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/development_guide.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/events.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/example.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/faq.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/freebsd_tuning.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/hash.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/howto_build_on_win32.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/install.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/license_copyright.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/load_balancing.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/nginx_dtrace_pid_provider.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/nginx.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_core_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_google_perftools_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_access_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_addition_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_auth_basic_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_auth_jwt_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_auth_request_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_autoindex_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_browser_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_charset_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_core_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_dav_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_empty_gif_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_f4f_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_fastcgi_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_flv_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_geoip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_geo_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_gunzip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_gzip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_gzip_static_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_headers_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_hls_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_image_filter_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_index_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_js_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_limit_conn_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_limit_req_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_log_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_map_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_memcached_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_mp4_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_perl_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_proxy_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_random_index_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_realip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_referer_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_rewrite_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_scgi_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_secure_link_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_session_log_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_slice_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_spdy_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_split_clients_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_ssi_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_ssl_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_status_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_stub_status_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_sub_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_upstream_conf_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_upstream_hc_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_upstream_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_userid_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_uwsgi_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_v2_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_http_xslt_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_auth_http_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_core_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_imap_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_pop3_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_proxy_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_smtp_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_mail_ssl_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_access_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_core_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_geoip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_geo_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_js_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_limit_conn_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_log_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_map_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_proxy_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_realip_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_return_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_split_clients_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_ssl_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_ssl_preread_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_upstream_hc_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/ngx_stream_upstream_module.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/njs_about.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/request_processing.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/server_names.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/stream_processing.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/switches.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/syntax.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/sys_errlist.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/syslog.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/variables_in_config.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/websocket.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/welcome_nginx_facebook.pod"
drop fakeroot "/usr/local/openresty/pod/nginx/windows.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_devel_kit-0.3.0/ngx_devel_kit-0.3.0.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_lua-0.10.8/.github.issue_template.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_lua-0.10.8/.github.pull_request_template.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_lua-0.10.8/ngx_lua-0.10.8.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_lua_upstream-0.06/ngx_lua_upstream-0.06.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_postgres-1.0rc7/ngx_postgres-1.0rc7.pod"
drop fakeroot "/usr/local/openresty/pod/ngx_postgres-1.0rc7/todo.pod"
drop fakeroot "/usr/local/openresty/pod/opm-0.0.3/opm-0.0.3.pod"
drop fakeroot "/usr/local/openresty/pod/rds-csv-nginx-module-0.07/rds-csv-nginx-module-0.07.pod"
drop fakeroot "/usr/local/openresty/pod/rds-json-nginx-module-0.14/rds-json-nginx-module-0.14.pod"
drop fakeroot "/usr/local/openresty/pod/redis2-nginx-module-0.14/redis2-nginx-module-0.14.pod"
drop fakeroot "/usr/local/openresty/pod/resty-cli-0.17/resty-cli-0.17.pod"
drop fakeroot "/usr/local/openresty/pod/set-misc-nginx-module-0.31/set-misc-nginx-module-0.31.pod"
drop fakeroot "/usr/local/openresty/pod/srcache-nginx-module-0.31/srcache-nginx-module-0.31.pod"
drop fakeroot "/usr/local/openresty/pod/xss-nginx-module-0.05/xss-nginx-module-0.05.pod"
drop fakeroot "/usr/local/openresty/resty.index"
drop fakeroot "/usr/sbin/usb_modeswitch"
drop fakeroot "/usr/sbin/usb_modeswitch_dispatcher"
drop fakeroot "/usr/share/usb_modeswitch/03f0:002a"
drop fakeroot "/usr/share/usb_modeswitch/0408:1000"
drop fakeroot "/usr/share/usb_modeswitch/0408:ea17"
drop fakeroot "/usr/share/usb_modeswitch/0408:ea25"
drop fakeroot "/usr/share/usb_modeswitch/0408:ea43"
drop fakeroot "/usr/share/usb_modeswitch/0408:f000"
drop fakeroot "/usr/share/usb_modeswitch/0408:f001"
drop fakeroot "/usr/share/usb_modeswitch/0421:060c"
drop fakeroot "/usr/share/usb_modeswitch/0421:0610"
drop fakeroot "/usr/share/usb_modeswitch/0421:0618"
drop fakeroot "/usr/share/usb_modeswitch/0421:061d"
drop fakeroot "/usr/share/usb_modeswitch/0421:0622"
drop fakeroot "/usr/share/usb_modeswitch/0421:0627"
drop fakeroot "/usr/share/usb_modeswitch/0421:062c"
drop fakeroot "/usr/share/usb_modeswitch/0421:0632"
drop fakeroot "/usr/share/usb_modeswitch/0421:0637"
drop fakeroot "/usr/share/usb_modeswitch/0471:1210:uMa=Philips"
drop fakeroot "/usr/share/usb_modeswitch/0471:1210:uMa=Wisue"
drop fakeroot "/usr/share/usb_modeswitch/0471:1237"
drop fakeroot "/usr/share/usb_modeswitch/0482:024d"
drop fakeroot "/usr/share/usb_modeswitch/04bb:bccd"
drop fakeroot "/usr/share/usb_modeswitch/04cc:2251"
drop fakeroot "/usr/share/usb_modeswitch/04cc:225c"
drop fakeroot "/usr/share/usb_modeswitch/04cc:226e"
drop fakeroot "/usr/share/usb_modeswitch/04cc:226f"
drop fakeroot "/usr/share/usb_modeswitch/04e8:680c"
drop fakeroot "/usr/share/usb_modeswitch/04e8:689a"
drop fakeroot "/usr/share/usb_modeswitch/04e8:f000:sMo=U209"
drop fakeroot "/usr/share/usb_modeswitch/04fc:2140"
drop fakeroot "/usr/share/usb_modeswitch/057c:62ff"
drop fakeroot "/usr/share/usb_modeswitch/057c:84ff"
drop fakeroot "/usr/share/usb_modeswitch/05c6:0010"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:sVe=GT"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:sVe=Option"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=AnyDATA"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=CELOT"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=Co.,Ltd"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=DGT"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=Option"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=SAMSUNG"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=SSE"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=StrongRising"
drop fakeroot "/usr/share/usb_modeswitch/05c6:1000:uMa=Vertex"
drop fakeroot "/usr/share/usb_modeswitch/05c6:2000"
drop fakeroot "/usr/share/usb_modeswitch/05c6:2001"
drop fakeroot "/usr/share/usb_modeswitch/05c6:6503"
drop fakeroot "/usr/share/usb_modeswitch/05c6:9024"
drop fakeroot "/usr/share/usb_modeswitch/05c6:f000"
drop fakeroot "/usr/share/usb_modeswitch/05c7:1000"
drop fakeroot "/usr/share/usb_modeswitch/0685:2000"
drop fakeroot "/usr/share/usb_modeswitch/072f:100d"
drop fakeroot "/usr/share/usb_modeswitch/07d1:a800"
drop fakeroot "/usr/share/usb_modeswitch/07d1:a804"
drop fakeroot "/usr/share/usb_modeswitch/0922:1001"
drop fakeroot "/usr/share/usb_modeswitch/0922:1003"
drop fakeroot "/usr/share/usb_modeswitch/0930:0d46"
drop fakeroot "/usr/share/usb_modeswitch/0ace:2011"
drop fakeroot "/usr/share/usb_modeswitch/0ace:20ff"
drop fakeroot "/usr/share/usb_modeswitch/0af0:4007"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6711"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6731"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6751"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6771"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6791"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6811"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6911"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6951"
drop fakeroot "/usr/share/usb_modeswitch/0af0:6971"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7011"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7031"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7051"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7071"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7111"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7211"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7251"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7271"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7301"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7311"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7361"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7381"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7401"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7501"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7601"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7701"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7706"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7801"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7901"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7a01"
drop fakeroot "/usr/share/usb_modeswitch/0af0:7a05"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8006"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8200"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8201"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8300"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8302"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8304"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8400"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8600"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8700"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8800"
drop fakeroot "/usr/share/usb_modeswitch/0af0:8900"
drop fakeroot "/usr/share/usb_modeswitch/0af0:9000"
drop fakeroot "/usr/share/usb_modeswitch/0af0:9200"
drop fakeroot "/usr/share/usb_modeswitch/0af0:c031"
drop fakeroot "/usr/share/usb_modeswitch/0af0:c100"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d001"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d013"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d031"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d033"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d035"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d055"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d057"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d058"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d155"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d157"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d255"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d257"
drop fakeroot "/usr/share/usb_modeswitch/0af0:d357"
drop fakeroot "/usr/share/usb_modeswitch/0b3c:c700"
drop fakeroot "/usr/share/usb_modeswitch/0b3c:f000"
drop fakeroot "/usr/share/usb_modeswitch/0b3c:f00c"
drop fakeroot "/usr/share/usb_modeswitch/0b3c:f017"
drop fakeroot "/usr/share/usb_modeswitch/0bdb:190d"
drop fakeroot "/usr/share/usb_modeswitch/0bdb:1910"
drop fakeroot "/usr/share/usb_modeswitch/0cf3:20ff"
drop fakeroot "/usr/share/usb_modeswitch/0d46:45a1"
drop fakeroot "/usr/share/usb_modeswitch/0d46:45a5"
drop fakeroot "/usr/share/usb_modeswitch/0df7:0800"
drop fakeroot "/usr/share/usb_modeswitch/0e8d:0002:uPr=MT"
drop fakeroot "/usr/share/usb_modeswitch/0e8d:0002:uPr=Product"
drop fakeroot "/usr/share/usb_modeswitch/0e8d:7109"
drop fakeroot "/usr/share/usb_modeswitch/0fca:8020"
drop fakeroot "/usr/share/usb_modeswitch/0fce:d0cf"
drop fakeroot "/usr/share/usb_modeswitch/0fce:d0df"
drop fakeroot "/usr/share/usb_modeswitch/0fce:d0e1"
drop fakeroot "/usr/share/usb_modeswitch/0fce:d103"
drop fakeroot "/usr/share/usb_modeswitch/0fd1:1000"
drop fakeroot "/usr/share/usb_modeswitch/1004:1000"
drop fakeroot "/usr/share/usb_modeswitch/1004:607f"
drop fakeroot "/usr/share/usb_modeswitch/1004:613a"
drop fakeroot "/usr/share/usb_modeswitch/1004:613f"
drop fakeroot "/usr/share/usb_modeswitch/1004:614e"
drop fakeroot "/usr/share/usb_modeswitch/1004:6156"
drop fakeroot "/usr/share/usb_modeswitch/1004:6190"
drop fakeroot "/usr/share/usb_modeswitch/1004:61aa"
drop fakeroot "/usr/share/usb_modeswitch/1004:61dd"
drop fakeroot "/usr/share/usb_modeswitch/1004:61e7"
drop fakeroot "/usr/share/usb_modeswitch/1004:61eb"
drop fakeroot "/usr/share/usb_modeswitch/1004:6327"
drop fakeroot "/usr/share/usb_modeswitch/1033:0035"
drop fakeroot "/usr/share/usb_modeswitch/106c:3b03"
drop fakeroot "/usr/share/usb_modeswitch/106c:3b05"
drop fakeroot "/usr/share/usb_modeswitch/106c:3b06"
drop fakeroot "/usr/share/usb_modeswitch/106c:3b11"
drop fakeroot "/usr/share/usb_modeswitch/106c:3b14"
drop fakeroot "/usr/share/usb_modeswitch/1076:7f40"
drop fakeroot "/usr/share/usb_modeswitch/109b:f009"
drop fakeroot "/usr/share/usb_modeswitch/10a9:606f"
drop fakeroot "/usr/share/usb_modeswitch/10a9:6080"
drop fakeroot "/usr/share/usb_modeswitch/1199:0fff"
drop fakeroot "/usr/share/usb_modeswitch/1266:1000"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1001"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1003"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1009"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1010"
drop fakeroot "/usr/share/usb_modeswitch/12d1:101e"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1030"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1031"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1413"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1414"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1446"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1449"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14ad"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14b5"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14b7"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14ba"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14c1"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14c3"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14c4"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14c5"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14d1"
drop fakeroot "/usr/share/usb_modeswitch/12d1:14fe"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1505"
drop fakeroot "/usr/share/usb_modeswitch/12d1:151a"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1520"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1521"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1523"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1526"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1553"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1557"
drop fakeroot "/usr/share/usb_modeswitch/12d1:155a"
drop fakeroot "/usr/share/usb_modeswitch/12d1:155b"
drop fakeroot "/usr/share/usb_modeswitch/12d1:156a"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1573"
drop fakeroot "/usr/share/usb_modeswitch/12d1:157c"
drop fakeroot "/usr/share/usb_modeswitch/12d1:157d"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1582"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1583"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1597"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15ca"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15cd"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15ce"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15cf"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15d0"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15d2"
drop fakeroot "/usr/share/usb_modeswitch/12d1:15e7"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1805"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1c0b"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1c1b"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1c24"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1d50"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1da1"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f01"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f02"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f03"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f07"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f09"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f11"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f15"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f16"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f17"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f18"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f19"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f1b"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f1c"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f1d"
drop fakeroot "/usr/share/usb_modeswitch/12d1:1f1e"
drop fakeroot "/usr/share/usb_modeswitch/12d1:380b"
drop fakeroot "/usr/share/usb_modeswitch/12d1:#android"
drop fakeroot "/usr/share/usb_modeswitch/12d1:#linux"
drop fakeroot "/usr/share/usb_modeswitch/1307:1169"
drop fakeroot "/usr/share/usb_modeswitch/1410:5010"
drop fakeroot "/usr/share/usb_modeswitch/1410:5020"
drop fakeroot "/usr/share/usb_modeswitch/1410:5023"
drop fakeroot "/usr/share/usb_modeswitch/1410:5030"
drop fakeroot "/usr/share/usb_modeswitch/1410:5031"
drop fakeroot "/usr/share/usb_modeswitch/1410:5041"
drop fakeroot "/usr/share/usb_modeswitch/1410:5055"
drop fakeroot "/usr/share/usb_modeswitch/1410:5059"
drop fakeroot "/usr/share/usb_modeswitch/1410:7001"
drop fakeroot "/usr/share/usb_modeswitch/1410:9020"
drop fakeroot "/usr/share/usb_modeswitch/1410:9022"
drop fakeroot "/usr/share/usb_modeswitch/148e:a000"
drop fakeroot "/usr/share/usb_modeswitch/148f:2578"
drop fakeroot "/usr/share/usb_modeswitch/15eb:7153"
drop fakeroot "/usr/share/usb_modeswitch/1614:0800"
drop fakeroot "/usr/share/usb_modeswitch/1614:0802"
drop fakeroot "/usr/share/usb_modeswitch/16d8:6281"
drop fakeroot "/usr/share/usb_modeswitch/16d8:6803"
drop fakeroot "/usr/share/usb_modeswitch/16d8:6804"
drop fakeroot "/usr/share/usb_modeswitch/16d8:700a"
drop fakeroot "/usr/share/usb_modeswitch/16d8:700b"
drop fakeroot "/usr/share/usb_modeswitch/16d8:f000"
drop fakeroot "/usr/share/usb_modeswitch/1726:f00e"
drop fakeroot "/usr/share/usb_modeswitch/1782:0003"
drop fakeroot "/usr/share/usb_modeswitch/198a:0003"
drop fakeroot "/usr/share/usb_modeswitch/198f:bccd"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0003"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0026"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0040"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0053"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0083:uPr=WCDMA"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0101"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0103"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0110"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0115"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0120"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0146"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0149"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0150"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0154"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0166"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0169"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0198"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0266"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0304"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0318"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0325"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0388"
drop fakeroot "/usr/share/usb_modeswitch/19d2:0413"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1001"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1007"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1009"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1013"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1017"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1030"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1038"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1171"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1175"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1179"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1201"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1207"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1210"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1216"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1219"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1224"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1225"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1227"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1232"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1233"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1237"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1238"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1420"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1511"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1514"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1517"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1520"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1523"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1528"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1536"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1542"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1580"
drop fakeroot "/usr/share/usb_modeswitch/19d2:1588"
drop fakeroot "/usr/share/usb_modeswitch/19d2:2000"
drop fakeroot "/usr/share/usb_modeswitch/19d2:2004"
drop fakeroot "/usr/share/usb_modeswitch/19d2:bccd"
drop fakeroot "/usr/share/usb_modeswitch/19d2:ffde"
drop fakeroot "/usr/share/usb_modeswitch/19d2:ffe6"
drop fakeroot "/usr/share/usb_modeswitch/19d2:fff5"
drop fakeroot "/usr/share/usb_modeswitch/19d2:fff6"
drop fakeroot "/usr/share/usb_modeswitch/19d2:#linux"
drop fakeroot "/usr/share/usb_modeswitch/1a8d:1000"
drop fakeroot "/usr/share/usb_modeswitch/1a8d:2000"
drop fakeroot "/usr/share/usb_modeswitch/1ab7:5700"
drop fakeroot "/usr/share/usb_modeswitch/1b7d:0700"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:000f"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:00ca"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:011f"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:022c"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:f000"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:f017"
drop fakeroot "/usr/share/usb_modeswitch/1bbb:f052"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:1001"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:6000"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:6061:uPr=Storage"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9101"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9200"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9401"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9800"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:98ff"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9d00"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9e00"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:9e08"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:f000"
drop fakeroot "/usr/share/usb_modeswitch/1c9e:f000:uMa=USB_Modem"
drop fakeroot "/usr/share/usb_modeswitch/1d09:1000"
drop fakeroot "/usr/share/usb_modeswitch/1d09:1021"
drop fakeroot "/usr/share/usb_modeswitch/1d09:1025"
drop fakeroot "/usr/share/usb_modeswitch/1da5:f000"
drop fakeroot "/usr/share/usb_modeswitch/1dbc:0669"
drop fakeroot "/usr/share/usb_modeswitch/1dd6:1000"
drop fakeroot "/usr/share/usb_modeswitch/1de1:1101"
drop fakeroot "/usr/share/usb_modeswitch/1e0e:f000"
drop fakeroot "/usr/share/usb_modeswitch/1e89:f000"
drop fakeroot "/usr/share/usb_modeswitch/1edf:6003"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0003"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0009"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0013"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0018"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0040"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0045"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:004a"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:004f"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0054"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0060"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0063"
drop fakeroot "/usr/share/usb_modeswitch/1ee8:0068"
drop fakeroot "/usr/share/usb_modeswitch/1f28:0021"
drop fakeroot "/usr/share/usb_modeswitch/1fac:0032"
drop fakeroot "/usr/share/usb_modeswitch/1fac:0130"
drop fakeroot "/usr/share/usb_modeswitch/1fac:0150"
drop fakeroot "/usr/share/usb_modeswitch/1fac:0151"
drop fakeroot "/usr/share/usb_modeswitch/2001:00a6"
drop fakeroot "/usr/share/usb_modeswitch/2001:98ff"
drop fakeroot "/usr/share/usb_modeswitch/2001:a401"
drop fakeroot "/usr/share/usb_modeswitch/2001:a403"
drop fakeroot "/usr/share/usb_modeswitch/2001:a405"
drop fakeroot "/usr/share/usb_modeswitch/2001:a706"
drop fakeroot "/usr/share/usb_modeswitch/2001:a707"
drop fakeroot "/usr/share/usb_modeswitch/2001:a708"
drop fakeroot "/usr/share/usb_modeswitch/2001:a805"
drop fakeroot "/usr/share/usb_modeswitch/2001:a80b"
drop fakeroot "/usr/share/usb_modeswitch/201e:1023"
drop fakeroot "/usr/share/usb_modeswitch/201e:2009"
drop fakeroot "/usr/share/usb_modeswitch/2020:0002"
drop fakeroot "/usr/share/usb_modeswitch/2020:f00e"
drop fakeroot "/usr/share/usb_modeswitch/2020:f00f"
drop fakeroot "/usr/share/usb_modeswitch/2077:1000"
drop fakeroot "/usr/share/usb_modeswitch/2077:f000"
drop fakeroot "/usr/share/usb_modeswitch/20a6:f00e"
drop fakeroot "/usr/share/usb_modeswitch/20b9:1682"
drop fakeroot "/usr/share/usb_modeswitch/21f5:1000"
drop fakeroot "/usr/share/usb_modeswitch/21f5:3010"
drop fakeroot "/usr/share/usb_modeswitch/2262:0001"
drop fakeroot "/usr/share/usb_modeswitch/22de:6801"
drop fakeroot "/usr/share/usb_modeswitch/22de:6803"
drop fakeroot "/usr/share/usb_modeswitch/22f4:0021"
drop fakeroot "/usr/share/usb_modeswitch/230d:0001"
drop fakeroot "/usr/share/usb_modeswitch/230d:0003"
drop fakeroot "/usr/share/usb_modeswitch/230d:0007"
drop fakeroot "/usr/share/usb_modeswitch/230d:000b"
drop fakeroot "/usr/share/usb_modeswitch/230d:000d"
drop fakeroot "/usr/share/usb_modeswitch/230d:0101"
drop fakeroot "/usr/share/usb_modeswitch/230d:0103"
drop fakeroot "/usr/share/usb_modeswitch/2357:0200"
drop fakeroot "/usr/share/usb_modeswitch/2357:f000"
drop fakeroot "/usr/share/usb_modeswitch/23a2:1010"
drop fakeroot "/usr/share/usb_modeswitch/257a:a000"
drop fakeroot "/usr/share/usb_modeswitch/257a:b000"
drop fakeroot "/usr/share/usb_modeswitch/257a:c000"
drop fakeroot "/usr/share/usb_modeswitch/257a:d000"
drop fakeroot "/usr/share/usb_modeswitch/8888:6500"
drop fakeroot "/usr/share/usb_modeswitch/ed09:1021"
upgrade
#!/bin/sh

#Drop EULA accept status
rm -Rf /data/configs/EULA/* > /dev/null 2>&1
## Upgrading done at here.

#Drop EULA
rm -Rf /data/configs/EULA/* > /dev/null 2>&1

#Update version.
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/share/lua/$LUA_VERSION/version.lua >> $logfile 2>&1
sed -i 's/SOFTWARE_VERSION=.*$/SOFTWARE_VERSION=[['$SWVER']]/g' /usr/local/openresty/lualib/version.lua >> $logfile 2>&1

echo "------------------------------------" >> $logfile
echo "Finished upgrading. [$WITH_ERROR errors, $WITH_WARNING warnnings.]" >> $logfile

rm -Rf $TARGET_TMP_DIR/_UPGRADE
sync
sync
exit $UPGRADE_RETURNS

